==> gds-2.17.3.tar.gz: gds-2.17.3/DMT/osc3/osc/StdAbove.cc <==
// Yo, Emacs! This is -*- C++ -*-
//
// $Id: StdAbove.cc 6619 2012-03-12 23:58:12Z john.zweizig@LIGO.ORG $
//
// Author: David Chin <dwchin@umich.edu>
//
//         2477 Randall Lab, Univ. of Michigan
//         500 E. University Ave.
//         Ann Arbor, MI 48109
//         +1-734-764-5146
//         +1-734-730-1274
//
// Written based on idea by K. Riles <kriles@umich.edu>

#include "osc/StdAbove.hh"

using namespace std;

osc::StdAbove::StdAbove(DaccAPI      *access,
                        ostream    *outstream,
                        string      chanName,
                        osc::IntParMap &intparams,
                        osc::DblParMap &dblparams,
                        int debug)
    : OperStateCond(access, outstream, chanName, "stdabove",
                    true, intparams, dblparams, debug)
{
    return;
}


bool osc::StdAbove::satisfied()
{
    if (mDebug > 0) {
        cout << "osc::" << mName << ": entered satisfied()" << endl;
    }

    // Read in time-series data
    TSeries *ptx = dataSeries();

    // If the time series is empty, return False
    if (ptx->getNSample() <= 0 || ptx->isEmpty() == true)
        return false;

    // If this condition has already been checked for in the current Frame,
    // return the buffered result
    Time t0 = ptx->getStartTime();
    if (t0 == mCurResults.mT0) {
        return mCurResults.mSatisfiedP;
    } else {

        updateResultsBuffer();
        
        //
        // Current stuff
        //
        mCurResults.mT0 = t0;

        try {
            readParamsFromChanMaybe();
        }

        catch (DataRead_err &err) {
            err.printErrMsg();
            return false;
        }

                
        // compute Standard deviation of time-series
        double avesqr = (*ptx)*(*ptx) / ptx->getNSample();
	double sqrave = pow(ptx->getAverage(),2.);
	double sqrdev = avesqr-sqrave;
	if (sqrdev>=0.0) {
	  mCurResults.mDoubleVal = sqrt(sqrdev);
	} else if (sqrdev>=-1.e-10*avesqr) {
	  mCurResults.mDoubleVal = 0.;
	} else {
	  cerr << "Warning from StdAbove: avesqr=" << avesqr << " sqrave=" << sqrave << " --> Imaginary standard deviation" << endl;
	  mCurResults.mDoubleVal = 0.;
	}

        if (mDebug > 0) {
            printf("osc::StdAbove on '%s': STD = %6.4f\n",
                   mChannel.c_str(), mCurResults.mDoubleVal);
        }

        if (mCurResults.mDoubleVal > mDblParams["threshold"].value())
            mCurResults.mSatisfiedP = true;
        else
            mCurResults.mSatisfiedP = false;

        return mCurResults.mSatisfiedP;
    }
}

==> gds-2.18.5.tar.gz: gds-2.18.5/DMT/osc3/osc/StdAbove.cc <==
// Yo, Emacs! This is -*- C++ -*-
//
// $Id: StdAbove.cc 7941 2017-10-04 02:22:45Z john.zweizig@LIGO.ORG $
//
// Author: David Chin <dwchin@umich.edu>
//
//         2477 Randall Lab, Univ. of Michigan
//         500 E. University Ave.
//         Ann Arbor, MI 48109
//         +1-734-764-5146
//         +1-734-730-1274
//
// Written based on idea by K. Riles <kriles@umich.edu>

#include "osc/StdAbove.hh"

using namespace std;

osc::StdAbove::StdAbove(DaccAPI      *access,
                        ostream    *outstream,
                        string      chanName,
                        osc::IntParMap &intparams,
                        osc::DblParMap &dblparams,
                        int debug)
    : OperStateCond(access, outstream, chanName, "stdabove",
                    true, intparams, dblparams, debug)
{
    return;
}


bool osc::StdAbove::satisfied()
{
    if (mDebug > 0) {
        cout << "osc::" << mName << ": entered satisfied()" << endl;
    }

    // Read in time-series data
    TSeries *ptx = dataSeries();

    // If the time series is empty, return False
    if (ptx->getNSample() <= 0 || ptx->isEmpty() == true)
        return false;

    // If this condition has already been checked for in the current Frame,
    // return the buffered result
    Time t0 = ptx->getStartTime();
    if (t0 == mCurResults.mT0) {
        return mCurResults.mSatisfiedP;
    } else {

        updateResultsBuffer();
        
        //
        // Current stuff
        //
        mCurResults.mT0 = t0;

        try {
            readParamsFromChanMaybe();
        }

        catch (DataRead_err &err) {
            err.printErrMsg();
            return false;
        }

                
        // compute Standard deviation of time-series
        double avesqr = (*ptx)*(*ptx) / ptx->getNSample();
	double sqrave = pow(ptx->getAverage(),2.);
	double sqrdev = avesqr-sqrave;
	if (sqrdev>=0.0) {
	  mCurResults.mDoubleVal = sqrt(sqrdev);
	} else if (sqrdev>=-1.e-10*avesqr) {
	  mCurResults.mDoubleVal = 0.;
	} else {
	  cerr << "Warning from StdAbove: avesqr=" << avesqr << " sqrave=" 
	       << sqrave << " --> Imaginary standard deviation" << endl;
	  mCurResults.mDoubleVal = 0.;
	}

        if (mDebug > 0) {
            printf("osc::StdAbove on '%s': STD = %6.4f\n",
                   mChannel.c_str(), mCurResults.mDoubleVal);
        }

        if (mCurResults.mDoubleVal > mDblParams["threshold"].value())
            mCurResults.mSatisfiedP = true;
        else
            mCurResults.mSatisfiedP = false;

        return mCurResults.mSatisfiedP;
    }
}
